@@ -59,7 +59,7 @@ static void *_package_changelog_open(alpm_pkg_t *pkg)
 	archive_read_support_format_all(archive);
 
 	if(archive_read_open_filename(archive, pkgfile,
-				ARCHIVE_DEFAULT_BYTES_PER_BLOCK) != ARCHIVE_OK) {
+				ALPM_BUFFER_SIZE) != ARCHIVE_OK) {
 		RET_ERR(pkg->handle, ALPM_ERR_PKG_OPEN, NULL);
 	}
 
@@ -390,7 +390,7 @@ alpm_pkg_t *_alpm_pkg_load_internal(alpm_handle_t *handle,
 	archive_read_support_format_all(archive);
 
 	if(archive_read_open_filename(archive, pkgfile,
-				ARCHIVE_DEFAULT_BYTES_PER_BLOCK) != ARCHIVE_OK) {
+				ALPM_BUFFER_SIZE) != ARCHIVE_OK) {
 		alpm_pkg_free(newpkg);
 		RET_ERR(handle, ALPM_ERR_PKG_OPEN, NULL);
 	}
@@ -482,17 +482,19 @@ alpm_pkg_t *_alpm_pkg_load_internal(alpm_handle_t *handle,
 	newpkg->origin_data.file = strdup(pkgfile);
 	newpkg->ops = get_file_pkg_ops();
 	newpkg->handle = handle;
+	newpkg->infolevel = INFRQ_BASE | INFRQ_DESC | INFRQ_SCRIPTLET;
 
 	if(full) {
-		/* attempt to hand back any memory we don't need */
-		files = realloc(files, sizeof(alpm_file_t) * files_count);
-		/* "checking for conflicts" requires a sorted list, ensure that here */
-		_alpm_log(handle, ALPM_LOG_DEBUG, "sorting package filelist for %s\n", pkgfile);
-		newpkg->files.files = files_msort(files, files_count);
+		if(files) {
+			/* attempt to hand back any memory we don't need */
+			files = realloc(files, sizeof(alpm_file_t) * files_count);
+			/* "checking for conflicts" requires a sorted list, ensure that here */
+			_alpm_log(handle, ALPM_LOG_DEBUG,
+					"sorting package filelist for %s\n", pkgfile);
+			newpkg->files.files = files_msort(files, files_count);
+		}
 		newpkg->files.count = files_count;
-		newpkg->infolevel = INFRQ_BASE | INFRQ_DESC | INFRQ_FILES | INFRQ_SCRIPTLET;
-	} else {
-		newpkg->infolevel = INFRQ_BASE | INFRQ_DESC | INFRQ_SCRIPTLET;
+		newpkg->infolevel |= INFRQ_FILES;
 	}
 
 	return newpkg;
