@@ -22,14 +22,8 @@
 
 #include "alpm.h"
 
-#ifdef PACMAN_DEBUG
-/* Log funtion entry points if debugging is enabled */
-#define ALPM_LOG_FUNC _alpm_log(PM_LOG_FUNCTION, "Enter %s\n", __func__)
-#else
-#define ALPM_LOG_FUNC
-#endif
-
-void _alpm_log(pmloglevel_t flag, const char *fmt, ...) __attribute__((format(printf,2,3)));
+void _alpm_log(alpm_handle_t *handle, alpm_loglevel_t flag,
+		const char *fmt, ...) __attribute__((format(printf,3,4)));
 
 #endif /* _ALPM_LOG_H */
 
