@@ -1,7 +1,7 @@
 /*
  *  signing.c
  *
- *  Copyright (c) 2008-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2008-2012 Pacman Development Team <pacman-dev@archlinux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -251,10 +251,24 @@ static int key_search(alpm_handle_t *handle, const char *fpr,
 	err = gpgme_get_key(ctx, fpr, &key, 0);
 	if(gpg_err_code(err) == GPG_ERR_EOF) {
 		_alpm_log(handle, ALPM_LOG_DEBUG, "key lookup failed, unknown key\n");
-		ret = 0;
-		goto error;
-	} else if(gpg_err_code(err) != GPG_ERR_NO_ERROR) {
-		_alpm_log(handle, ALPM_LOG_DEBUG, "gpg error: %s\n", gpgme_strerror(err));
+		/* Try an alternate lookup using the 8 character fingerprint value, since
+		 * busted-ass keyservers can't support lookups using subkeys with the full
+		 * value as of now. This is why 2012 is not the year of PGP encryption. */
+		if(strlen(fpr) > 8) {
+			const char *short_fpr = fpr + strlen(fpr) - 8;
+			_alpm_log(handle, ALPM_LOG_DEBUG,
+					"looking up key %s remotely\n", short_fpr);
+			err = gpgme_get_key(ctx, short_fpr, &key, 0);
+			if(gpg_err_code(err) == GPG_ERR_EOF) {
+				_alpm_log(handle, ALPM_LOG_DEBUG, "key lookup failed, unknown key\n");
+				ret = 0;
+			}
+		} else {
+			ret = 0;
+		}
+	}
+
+	if(gpg_err_code(err) != GPG_ERR_NO_ERROR) {
 		goto error;
 	}
 
@@ -270,9 +284,11 @@ static int key_search(alpm_handle_t *handle, const char *fpr,
 	pgpkey->email = key->uids->email;
 	pgpkey->created = key->subkeys->timestamp;
 	pgpkey->expires = key->subkeys->expires;
-	ret = 1;
+	gpgme_release(ctx);
+	return 1;
 
 error:
+	_alpm_log(handle, ALPM_LOG_DEBUG, "gpg error: %s\n", gpgme_strerror(err));
 	gpgme_release(ctx);
 	return ret;
 }
