@@ -223,7 +223,7 @@ static void version(void)
 {
 	printf("\n");
 	printf(" .--.                  Pacman v%s - libalpm v%s\n", PACKAGE_VERSION, alpm_version());
-	printf("/ _.-' .-.  .-.  .-.   Copyright (C) 2006-2011 Pacman Development Team\n");
+	printf("/ _.-' .-.  .-.  .-.   Copyright (C) 2006-2012 Pacman Development Team\n");
 	printf("\\  '-. '-'  '-'  '-'   Copyright (C) 2002-2006 Judd Vinet\n");
 	printf(" '--'\n");
 	printf(_("                       This program may be freely redistributed under\n"
@@ -726,7 +726,9 @@ static int parseargs(int argc, char *argv[])
 	return 0;
 }
 
-/** print commandline to logfile
+/** Print command line to logfile.
+ * @param argc
+ * @param argv
  */
 static void cl_to_log(int argc, char* argv[])
 {
@@ -754,8 +756,8 @@ static void cl_to_log(int argc, char* argv[])
 }
 
 /** Main function.
- * @param argc argc
- * @param argv argv
+ * @param argc
+ * @param argv
  * @return A return code indicating success, failure, etc.
  */
 int main(int argc, char *argv[])
@@ -801,7 +803,7 @@ int main(int argc, char *argv[])
 	config = config_new();
 
 	/* disable progressbar if the output is redirected */
-	if(!isatty(1)) {
+	if(!isatty(fileno(stdout))) {
 		config->noprogressbar = 1;
 	}
 
@@ -914,7 +916,7 @@ int main(int argc, char *argv[])
 		list_display("Targets   :", pm_targets);
 	}
 
-	/* Log commandline */
+	/* Log command line */
 	if(needs_root()) {
 		cl_to_log(argc, argv);
 	}
