@@ -1,7 +1,7 @@
 /*
  *  util.c
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2012 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -192,10 +192,10 @@ int rmrf(const char *path)
 			return 1;
 		}
 		for(dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
-			if(dp->d_ino) {
-				char name[PATH_MAX];
-				sprintf(name, "%s/%s", path, dp->d_name);
+			if(dp->d_name) {
 				if(strcmp(dp->d_name, "..") != 0 && strcmp(dp->d_name, ".") != 0) {
+					char name[PATH_MAX];
+					snprintf(name, PATH_MAX, "%s/%s", path, dp->d_name);
 					errflag += rmrf(name);
 				}
 			}
@@ -312,19 +312,6 @@ void indentprint(const char *str, size_t indent)
 	free(wcstr);
 }
 
-/* Convert a string to uppercase
- */
-char *strtoupper(char *str)
-{
-	char *ptr = str;
-
-	while(*ptr) {
-		(*ptr) = (char)toupper((unsigned char)*ptr);
-		ptr++;
-	}
-	return str;
-}
-
 /* Trim whitespace and newlines from a string
  */
 char *strtrim(char *str)
@@ -804,13 +791,13 @@ static alpm_list_t *create_verbose_row(pm_target_t *target, int dl_size)
 	/* and size */
 	size -= target->remove ? alpm_pkg_get_isize(target->remove) : 0;
 	size += target->install ? alpm_pkg_get_isize(target->install) : 0;
-	human_size = humanize_size(size, 'M', &label);
+	human_size = humanize_size(size, 'M', 2, &label);
 	pm_asprintf(&str, "%.2f %s", human_size, label);
 	ret = alpm_list_add(ret, str);
 
 	if(dl_size) {
 		size = target->install ? alpm_pkg_download_size(target->install) : 0;
-		human_size = humanize_size(size, 'M', &label);
+		human_size = humanize_size(size, 'M', 2, &label);
 		if(size != 0) {
 			pm_asprintf(&str, "%.2f %s", human_size, label);
 		} else {
@@ -848,8 +835,12 @@ static void _display_targets(alpm_list_t *targets, int verbose)
 			/* add up size of all removed packages */
 			rsize += alpm_pkg_get_isize(target->remove);
 		}
+	}
+
+	/* form data for both verbose and non-verbose display */
+	for(i = targets; i; i = alpm_list_next(i)) {
+		pm_target_t *target = i->data;
 
-		/* form data for both verbose and non-verbose display */
 		rows = alpm_list_add(rows, create_verbose_row(target, show_dl_size));
 		if(target->install) {
 			pm_asprintf(&str, "%s-%s", alpm_pkg_get_name(target->install),
@@ -890,21 +881,21 @@ static void _display_targets(alpm_list_t *targets, int verbose)
 	free(str);
 
 	if(dlsize > 0 || config->op_s_downloadonly) {
-		size = humanize_size(dlsize, 'M', &label);
+		size = humanize_size(dlsize, 'M', 2, &label);
 		printf(_("Total Download Size:    %.2f %s\n"), size, label);
 	}
 	if(!config->op_s_downloadonly) {
 		if(isize > 0) {
-			size = humanize_size(isize, 'M', &label);
+			size = humanize_size(isize, 'M', 2, &label);
 			printf(_("Total Installed Size:   %.2f %s\n"), size, label);
 		}
 		if(rsize > 0 && isize == 0) {
-			size = humanize_size(rsize, 'M', &label);
+			size = humanize_size(rsize, 'M', 2, &label);
 			printf(_("Total Removed Size:     %.2f %s\n"), size, label);
 		}
 		/* only show this net value if different from raw installed size */
 		if(isize > 0 && rsize > 0) {
-			size = humanize_size(isize - rsize, 'M', &label);
+			size = humanize_size(isize - rsize, 'M', 2, &label);
 			printf(_("Net Upgrade Size:       %.2f %s\n"), size, label);
 		}
 	}
@@ -997,18 +988,31 @@ static char *pkg_get_location(alpm_pkg_t *pkg)
 	}
 }
 
+/* a pow() implementation that is specialized for an integer base and small,
+ * positive-only integer exponents. */
+static double simple_pow(int base, int exp)
+{
+	double result = 1.0;
+	for(; exp > 0; exp--) {
+		result *= base;
+	}
+	return result;
+}
+
 /** Converts sizes in bytes into human readable units.
  *
  * @param bytes the size in bytes
  * @param target_unit '\0' or a short label. If equal to one of the short unit
  * labels ('B', 'K', ...) bytes is converted to target_unit; if '\0', the first
  * unit which will bring the value to below a threshold of 2048 will be chosen.
- * @param long_labels whether to use short ("K") or long ("KiB") unit labels
+ * @param precision number of decimal places, ensures -0.00 gets rounded to
+ * 0.00; -1 if no rounding desired
  * @param label will be set to the appropriate unit label
  *
  * @return the size in the appropriate unit
  */
-double humanize_size(off_t bytes, const char target_unit, const char **label)
+double humanize_size(off_t bytes, const char target_unit, int precision,
+		const char **label)
 {
 	static const char *labels[] = {"B", "KiB", "MiB", "GiB",
 		"TiB", "PiB", "EiB", "ZiB", "YiB"};
@@ -1030,6 +1034,12 @@ double humanize_size(off_t bytes, const char target_unit, const char **label)
 		*label = labels[index];
 	}
 
+	/* fix FS#27924 so that it doesn't display negative zeroes */
+	if(precision >= 0 && val < 0.0 &&
+			val > (-0.5 / simple_pow(10, precision))) {
+		val = 0.0;
+	}
+
 	return val;
 }
 
@@ -1378,6 +1388,12 @@ static int question(short preset, char *fmt, va_list args)
 			return preset;
 		}
 
+		/* if stdin is piped, response does not get printed out, and as a result
+		 * a \n is missing, resulting in broken output (FS#27909) */
+		if(!isatty(fileno(stdin))) {
+			fprintf(stream, "%s\n", response);
+		}
+
 		if(strcasecmp(response, _("Y")) == 0 || strcasecmp(response, _("YES")) == 0) {
 			return 1;
 		} else if(strcasecmp(response, _("N")) == 0 || strcasecmp(response, _("NO")) == 0) {
