@@ -1,7 +1,7 @@
 #! /usr/bin/python
 #
 #  Copyright (c) 2006 by Aurelien Foret <orelien@chez.com>
-# 
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -17,7 +17,6 @@
 
 
 import os
-import os.path
 
 import pmtest
 
@@ -102,35 +101,39 @@ class pmenv(object):
                 tfailed.append(test)
 
         def _printtest(t):
-            success = test.result["success"]
-            fail = test.result["fail"]
-            rules = len(test.rules)
+            success = t.result["success"]
+            fail = t.result["fail"]
+            rules = len(t.rules)
             if fail == 0:
                 result = "[PASS]"
             else:
                 result = "[FAIL]"
             print result,
             print "%s Rules: OK = %2u  FAIL = %2u  SKIP = %2u" \
-                    % (test.testname.ljust(34), success, fail, \
+                    % (t.testname.ljust(34), success, fail, \
                        rules - (success + fail))
             if fail != 0:
                 # print test description if test failed
-                print "      ", test.description
+                print "      ", t.description
 
         print "=========="*8
         print "Results"
         print "----------"*8
         print " Passed:"
-        for test in tpassed: _printtest(test)
+        for test in tpassed:
+            _printtest(test)
         print "----------"*8
         print " Expected Failures:"
-        for test in texpectedfail: _printtest(test)
+        for test in texpectedfail:
+            _printtest(test)
         print "----------"*8
         print " Unexpected Passes:"
-        for test in tunexpectedpass: _printtest(test)
+        for test in tunexpectedpass:
+            _printtest(test)
         print "----------"*8
         print " Failed:"
-        for test in tfailed: _printtest(test)
+        for test in tfailed:
+            _printtest(test)
         print "----------"*8
 
         total = len(self.testcases)
@@ -142,7 +145,4 @@ class pmenv(object):
             print "Fail             = %3u (%6.2f%%)" % (self.failed, float(self.failed) * 100 / total)
         print ""
 
-if __name__ == "__main__":
-    pass
-
 # vim: set ts=4 sw=4 et:
